@@ -62,6 +62,8 @@ struct _TrackerConfigMigrationEntry {
 	const gchar *file_section;
 	const gchar *file_key;
 	const gchar *settings_key;
+	gboolean is_dir_list;
+	gboolean is_dir_list_recursive;
 };
 
 GType              tracker_config_file_get_type           (void) G_GNUC_CONST;
@@ -72,6 +74,9 @@ gboolean           tracker_config_file_migrate            (TrackerConfigFile
 gboolean           tracker_config_file_import_to_settings (TrackerConfigFile           *file,
                                                            GSettings                   *settings,
                                                            TrackerConfigMigrationEntry *entries);
+gboolean           tracker_config_file_load_from_file     (TrackerConfigFile           *file,
+                                                           GObject                     *object,
+                                                           TrackerConfigMigrationEntry *entries);
 gboolean           tracker_config_file_save               (TrackerConfigFile           *file);
 
 G_END_DECLS
