@@ -20,6 +20,7 @@ tracker_directory_flags_get_type (void)
       { TRACKER_DIRECTORY_FLAG_MONITOR, "TRACKER_DIRECTORY_FLAG_MONITOR", "monitor" },
       { TRACKER_DIRECTORY_FLAG_IGNORE, "TRACKER_DIRECTORY_FLAG_IGNORE", "ignore" },
       { TRACKER_DIRECTORY_FLAG_PRESERVE, "TRACKER_DIRECTORY_FLAG_PRESERVE", "preserve" },
+      { TRACKER_DIRECTORY_FLAG_PRIORITY, "TRACKER_DIRECTORY_FLAG_PRIORITY", "priority" },
       { 0, NULL, NULL }
     };
     GType g_define_type_id = 
@@ -72,6 +73,30 @@ tracker_filter_policy_get_type (void)
   return g_define_type_id__volatile;
 }
 
+GType
+tracker_network_type_get_type (void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+ 
+  if (g_once_init_enter (&g_define_type_id__volatile)) {
+    static const GEnumValue values[] = {
+      { TRACKER_NETWORK_TYPE_NONE, "TRACKER_NETWORK_TYPE_NONE", "none" },
+      { TRACKER_NETWORK_TYPE_UNKNOWN, "TRACKER_NETWORK_TYPE_UNKNOWN", "unknown" },
+      { TRACKER_NETWORK_TYPE_GPRS, "TRACKER_NETWORK_TYPE_GPRS", "gprs" },
+      { TRACKER_NETWORK_TYPE_EDGE, "TRACKER_NETWORK_TYPE_EDGE", "edge" },
+      { TRACKER_NETWORK_TYPE_3G, "TRACKER_NETWORK_TYPE_3G", "3g" },
+      { TRACKER_NETWORK_TYPE_LAN, "TRACKER_NETWORK_TYPE_LAN", "lan" },
+      { 0, NULL, NULL }
+    };
+    GType g_define_type_id = 
+       g_enum_register_static (g_intern_static_string ("TrackerNetworkType"), values);
+      
+    g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+  }
+    
+  return g_define_type_id__volatile;
+}
+
 
 
 /* Generated data ends here */
