@@ -41,6 +41,7 @@ G_BEGIN_DECLS
  * @TRACKER_DIRECTORY_FLAG_MONITOR: Should setup monitors in the items found in the directory.
  * @TRACKER_DIRECTORY_FLAG_IGNORE: Should ignore the directory contents.
  * @TRACKER_DIRECTORY_FLAG_PRESERVE: Should preserve items in the directory even if the directory gets removed.
+ * @TRACKER_DIRECTORY_FLAG_PRIORITY: Internally a priority queue is used and this flag makes sure the directory is given a priority over other directories queued.
  *
  * Flags used when adding a new directory to be indexed in the #TrackerIndexingTree.
  */
@@ -50,7 +51,8 @@ typedef enum {
 	TRACKER_DIRECTORY_FLAG_CHECK_MTIME = 1 << 2,
 	TRACKER_DIRECTORY_FLAG_MONITOR     = 1 << 3,
 	TRACKER_DIRECTORY_FLAG_IGNORE      = 1 << 4,
-	TRACKER_DIRECTORY_FLAG_PRESERVE    = 1 << 5
+	TRACKER_DIRECTORY_FLAG_PRESERVE    = 1 << 5,
+	TRACKER_DIRECTORY_FLAG_PRIORITY    = 1 << 6
 } TrackerDirectoryFlags;
 
 /**
@@ -79,6 +81,34 @@ typedef enum {
 	TRACKER_FILTER_POLICY_ACCEPT
 } TrackerFilterPolicy;
 
+/**
+ * TrackerNetworkType:
+ * @TRACKER_NETWORK_TYPE_NONE: Network is disconnected
+ * @TRACKER_NETWORK_TYPE_UNKNOWN: Network status is unknown
+ * @TRACKER_NETWORK_TYPE_GPRS: Network is connected over a GPRS
+ * connection
+ * @TRACKER_NETWORK_TYPE_EDGE: Network is connected over an EDGE
+ * connection
+ * @TRACKER_NETWORK_TYPE_3G: Network is connected over a 3G or
+ * faster (HSDPA, UMTS, ...) connection
+ * @TRACKER_NETWORK_TYPE_LAN: Network is connected over a local
+ * network connection. This can be ethernet, wifi, etc.
+ *
+ * Enumerates the different types of connections that the device might
+ * use when connected to internet. Note that not all providers might
+ * provide this information.
+ *
+ * Since: 0.18
+ **/
+typedef enum {
+	TRACKER_NETWORK_TYPE_NONE,
+	TRACKER_NETWORK_TYPE_UNKNOWN,
+	TRACKER_NETWORK_TYPE_GPRS,
+	TRACKER_NETWORK_TYPE_EDGE,
+	TRACKER_NETWORK_TYPE_3G,
+	TRACKER_NETWORK_TYPE_LAN
+} TrackerNetworkType;
+
 G_END_DECLS
 
 #endif /* __TRACKER_MINER_ENUMS_H__ */
