@@ -22,18 +22,11 @@
 
 #define __LIBTRACKER_MINER_H_INSIDE__
 
-#include <libtracker-miner/tracker-crawler.h>
 #include <libtracker-miner/tracker-decorator.h>
 #include <libtracker-miner/tracker-decorator-fs.h>
-#include <libtracker-miner/tracker-storage.h>
-#include <libtracker-miner/tracker-thumbnailer.h>
-#include <libtracker-miner/tracker-media-art.h>
-#include <libtracker-miner/tracker-network-provider.h>
-#include <libtracker-miner/tracker-password-provider.h>
 #include <libtracker-miner/tracker-miner-object.h>
+#include <libtracker-miner/tracker-miner-online.h>
 #include <libtracker-miner/tracker-miner-fs.h>
-#include <libtracker-miner/tracker-miner-web.h>
-#include <libtracker-miner/tracker-miner-manager.h>
 #include <libtracker-miner/tracker-miner-enums.h>
 #include <libtracker-miner/tracker-miner-enum-types.h>
 #include <libtracker-miner/tracker-indexing-tree.h>
